@@ -19,9 +19,9 @@ from moose_lib.dmv2 import (
     get_apis,
     get_sql_resources,
     get_workflows,
+    get_web_apps,
     OlapTable,
-    View,
-    MaterializedView,
+    OlapConfig,
     SqlResource
 )
 from moose_lib.dmv2.stream import KafkaSchemaConfig
@@ -128,6 +128,45 @@ class S3QueueConfigDict(BaseEngineConfigDict):
     headers: Optional[Dict[str, str]] = None
 
 
+class S3ConfigDict(BaseEngineConfigDict):
+    """Configuration for S3 engine."""
+    engine: Literal["S3"] = "S3"
+    path: str
+    format: str
+    aws_access_key_id: Optional[str] = None
+    aws_secret_access_key: Optional[str] = None
+    compression: Optional[str] = None
+    partition_strategy: Optional[str] = None
+    partition_columns_in_data_file: Optional[str] = None
+
+
+class BufferConfigDict(BaseEngineConfigDict):
+    """Configuration for Buffer engine."""
+    engine: Literal["Buffer"] = "Buffer"
+    target_database: str
+    target_table: str
+    num_layers: int
+    min_time: int
+    max_time: int
+    min_rows: int
+    max_rows: int
+    min_bytes: int
+    max_bytes: int
+    flush_time: Optional[int] = None
+    flush_rows: Optional[int] = None
+    flush_bytes: Optional[int] = None
+
+
+class DistributedConfigDict(BaseEngineConfigDict):
+    """Configuration for Distributed engine."""
+    engine: Literal["Distributed"] = "Distributed"
+    cluster: str
+    target_database: str
+    target_table: str
+    sharding_key: Optional[str] = None
+    policy_name: Optional[str] = None
+
+
 # Discriminated union of all engine configurations
 EngineConfigDict = Union[
     MergeTreeConfigDict,
@@ -138,7 +177,10 @@ EngineConfigDict = Union[
     ReplicatedReplacingMergeTreeConfigDict,
     ReplicatedAggregatingMergeTreeConfigDict,
     ReplicatedSummingMergeTreeConfigDict,
-    S3QueueConfigDict
+    S3QueueConfigDict,
+    S3ConfigDict,
+    BufferConfigDict,
+    DistributedConfigDict
 ]
 
 
@@ -150,6 +192,7 @@ class TableConfig(BaseModel):
         columns: List of columns with their types and attributes.
         order_by: List of columns used for the ORDER BY clause.
         partition_by: The column name used for the PARTITION BY clause.
+        sample_by_expression: Optional SAMPLE BY expression for data sampling.
         engine_config: Engine configuration with type-safe, engine-specific parameters.
         version: Optional version string of the table configuration.
         metadata: Optional metadata for the table.
@@ -162,11 +205,15 @@ class TableConfig(BaseModel):
     columns: List[Column]
     order_by: List[str] | str
     partition_by: Optional[str]
+    sample_by_expression: Optional[str] = None
     engine_config: Optional[EngineConfigDict] = Field(None, discriminator='engine')
     version: Optional[str] = None
     metadata: Optional[dict] = None
     life_cycle: Optional[str] = None
-    table_settings: Optional[Dict[str, str]] = None
+    table_settings: Optional[dict[str, str]] = None
+    indexes: list[OlapConfig.TableIndex] = []
+    ttl: Optional[str] = None
+    database: Optional[str] = None
 
 
 class TopicConfig(BaseModel):
@@ -265,6 +312,32 @@ class WorkflowJson(BaseModel):
     schedule: Optional[str] = None
 
 
+class WebAppMetadataJson(BaseModel):
+    """Internal representation of WebApp metadata for serialization.
+
+    Attributes:
+        description: Optional description of the WebApp.
+    """
+    model_config = model_config
+
+    description: Optional[str] = None
+
+
+class WebAppJson(BaseModel):
+    """Internal representation of a WebApp configuration for serialization.
+
+    Attributes:
+        name: Name of the WebApp.
+        mount_path: The URL path where the WebApp is mounted.
+        metadata: Optional metadata for documentation purposes.
+    """
+    model_config = model_config
+
+    name: str
+    mount_path: str
+    metadata: Optional[WebAppMetadataJson] = None
+
+
 class InfrastructureSignatureJson(BaseModel):
     """Represents the unique signature of an infrastructure component (Table, Topic, etc.).
 
@@ -311,6 +384,7 @@ class InfrastructureMap(BaseModel):
         apis: Dictionary mapping API names to their configurations.
         sql_resources: Dictionary mapping SQL resource names to their configurations.
         workflows: Dictionary mapping workflow names to their configurations.
+        web_apps: Dictionary mapping WebApp names to their configurations.
     """
     model_config = model_config
 
@@ -320,6 +394,7 @@ class InfrastructureMap(BaseModel):
     apis: dict[str, InternalApiConfig]
     sql_resources: dict[str, SqlResourceConfig]
     workflows: dict[str, WorkflowJson]
+    web_apps: dict[str, WebAppJson]
 
 
 def _map_sql_resource_ref(r: Any) -> InfrastructureSignatureJson:
@@ -431,7 +506,7 @@ def _convert_engine_instance_to_config_dict(engine: "EngineConfig") -> EngineCon
     Returns:
         EngineConfigDict with engine-specific configuration
     """
-    from moose_lib.blocks import S3QueueEngine
+    from moose_lib.blocks import S3QueueEngine, S3Engine, BufferEngine, DistributedEngine
 
     # Try S3Queue first
     if isinstance(engine, S3QueueEngine):
@@ -443,6 +518,45 @@ def _convert_engine_instance_to_config_dict(engine: "EngineConfig") -> EngineCon
             compression=engine.compression,
             headers=engine.headers
         )
+    
+    # Try S3
+    if isinstance(engine, S3Engine):
+        return S3ConfigDict(
+            path=engine.path,
+            format=engine.format,
+            aws_access_key_id=engine.aws_access_key_id,
+            aws_secret_access_key=engine.aws_secret_access_key,
+            compression=engine.compression,
+            partition_strategy=engine.partition_strategy,
+            partition_columns_in_data_file=engine.partition_columns_in_data_file
+        )
+    
+    # Try Buffer
+    if isinstance(engine, BufferEngine):
+        return BufferConfigDict(
+            target_database=engine.target_database,
+            target_table=engine.target_table,
+            num_layers=engine.num_layers,
+            min_time=engine.min_time,
+            max_time=engine.max_time,
+            min_rows=engine.min_rows,
+            max_rows=engine.max_rows,
+            min_bytes=engine.min_bytes,
+            max_bytes=engine.max_bytes,
+            flush_time=engine.flush_time,
+            flush_rows=engine.flush_rows,
+            flush_bytes=engine.flush_bytes
+        )
+    
+    # Try Distributed
+    if isinstance(engine, DistributedEngine):
+        return DistributedConfigDict(
+            cluster=engine.cluster,
+            target_database=engine.target_database,
+            target_table=engine.target_table,
+            sharding_key=engine.sharding_key,
+            policy_name=engine.policy_name
+        )
 
     # Try basic engines
     basic_config = _convert_basic_engine_instance(engine)
@@ -538,6 +652,7 @@ def to_infra_map() -> dict:
     apis = {}
     sql_resources = {}
     workflows = {}
+    web_apps = {}
 
     for _registry_key, table in get_tables().items():
         # Convert engine configuration to new format
@@ -571,12 +686,16 @@ def to_infra_map() -> dict:
             columns=table._column_list,
             order_by=order_by_value,
             partition_by=table.config.partition_by,
+            sample_by_expression=table.config.sample_by_expression,
             engine_config=engine_config,
             version=table.config.version,
             metadata=getattr(table, "metadata", None),
             life_cycle=table.config.life_cycle.value if table.config.life_cycle else None,
             # Map 'settings' to 'table_settings' for internal use
             table_settings=table_settings if table_settings else None,
+            indexes=table.config.indexes,
+            ttl=table.config.ttl,
+            database=table.config.database,
         )
 
     for name, stream in get_streams().items():
@@ -657,16 +776,30 @@ def to_infra_map() -> dict:
             schedule=workflow.config.schedule,
         )
 
+    for name, web_app in get_web_apps().items():
+        mount_path = web_app.config.mount_path or "/"
+        metadata = None
+        if web_app.config.metadata:
+            metadata = WebAppMetadataJson(
+                description=web_app.config.metadata.description
+            )
+        web_apps[name] = WebAppJson(
+            name=web_app.name,
+            mount_path=mount_path,
+            metadata=metadata,
+        )
+
     infra_map = InfrastructureMap(
         tables=tables,
         topics=topics,
         ingest_apis=ingest_apis,
         apis=apis,
         sql_resources=sql_resources,
-        workflows=workflows
+        workflows=workflows,
+        web_apps=web_apps
     )
 
-    return infra_map.model_dump(by_alias=True)
+    return infra_map.model_dump(by_alias=True, exclude_none=False)
 
 
 def load_models():
@@ -682,7 +815,9 @@ def load_models():
        (`___MOOSE_STUFF___start` and `end___MOOSE_STUFF___`), which the
        calling system uses to extract the configuration.
     """
-    import_module("app.main")
+    import os
+    source_dir = os.environ.get("MOOSE_SOURCE_DIR", "app")
+    import_module(f"{source_dir}.main")
 
     # Generate the infrastructure map
     infra_map = to_infra_map()
